# tools/user/autodetect.py
#
# Autodetects serial ports on Linux and Mac.
#
# Copyright 2011 Rice University.
#
# This file is part of the Owl Embedded Python System and is provided under
# the MIT open-source license. See the LICENSE and COPYING files for details
# about your rights to use, modify, and distribute Owl.


import sys
import os
import itertools
import re

# Windows side
if 'win32' in sys.platform:
    import _winreg as winreg

    class IterationError(Exception):
        pass

    def enumerate_serial_ports():
        """ Uses the Win32 registry to return an
            iterator of serial (COM) ports
            existing on this computer.
        """
        path = 'HARDWARE\\DEVICEMAP\\SERIALCOMM'
        try:
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path)
        except WindowsError:
            raise IterationError, "no serial ports found, error opening registry key"
            
        for i in itertools.count():
            try:
                yield winreg.EnumValue(key, i)
            except EnvironmentError:
                break
              
    # For ports of the form CNCA... make sure that pyserial can read them
    def full_port_name(portname):
        """ Given a port-name (of the form COM7,
            COM12, CNCA0, etc.) returns a full
            name suitable for opening with the
            Serial class.
        """
        m = re.match('^COM(\d+)$', portname)
        if m and int(m.group(1)) < 1000:
            return portname
        return '\\\\.\\' + portname

class AutodetectError(Exception):
    pass
    
def autodetect():
    if 'arwin' in sys.platform or 'linux' in sys.platform:
        devs = os.listdir('/dev/')

        if 'mcu' in devs:
            return '/dev/mcu'

        if 'arwin' in sys.platform:
            devs = ['/dev/' + d for d in devs if 'tty.usb' in d]
        elif 'linux' in sys.platform and 'serial' in devs:
            serialDevs = os.listdir('/dev/serial/by-id/')
            devs = ['/dev/serial/by-id/' + d for d in serialDevs if 'FTDI_FT232R' in d]
        else:
            devs = ['/dev/' + d for d in devs if 'ttyUSB' in d]

        if len(devs) > 1:
            raise AutodetectError, "multiple USB serial devices found"
        if len(devs) < 1:
            raise AutodetectError, "no USB serial devices found"
        else:
            return devs[0]
    elif 'win' in sys.platform:
        # Get all of the ports from the system in their full form
        ports = []
        try:
            for port in enumerate_serial_ports():
                # Make sure it is a Virtual COM Port (VCP), VCP in the name
                if 'VCP' in port[0]:
                    ports.append(full_port_name(str(port[1])))
        except IterationError:
            raise AutodetectError, "no virtual COM ports found"
           
        # Return the first one, but if there are multiple ports, warn the user about it
        if len(ports) > 1:
            print "WARNING: Multiple serial devices found (" + ", ".join(ports) + "). Trying the first one..."
            return ports[0]
        elif len(ports) < 1:
            raise AutodetectError, "no serial devices found"
        else:
            return ports[0]
    else:
        raise AutodetectError, "cannot autodetect on platform %s" % sys.platform

if __name__ == "__main__":
    print "autodetected: %s" % autodetect()
